-- -------------------------------------------------------------------------- --
-- GridIndicatorSidePlus by kunda                                             --
-- -------------------------------------------------------------------------- --

local GridFrame = Grid:GetModule("GridFrame")
local L = GridIndicatorSidePlus_Locales
local AceOO = AceLibrary("AceOO-2.0")
local configMode = false

local GridIndicatorSidePlus = Grid:GetModule("GridStatus"):NewModule("GridIndicatorSidePlus")

GridIndicatorSidePlus.defaultDB = {
	SidePlusSpace = 8,
	SidePlusSize = 5,
	SidePlusOriginalSize = true,
	SidePlusOffset = 1,
}

local indicators = GridFrame.frameClass.prototype.indicators
table.insert(indicators, { type = "sidePlusTleft",   order = 15.1, name = L["Top Side (left)"] })
table.insert(indicators, { type = "sidePlusTcenter", order = 15.2, name = L["Top Side (center)"] })
table.insert(indicators, { type = "sidePlusTright",  order = 15.3, name = L["Top Side (right)"] })
table.insert(indicators, { type = "sidePlusBleft",   order = 16.1, name = L["Bottom Side (left)"] })
table.insert(indicators, { type = "sidePlusBcenter", order = 16.2, name = L["Bottom Side (center)"] })
table.insert(indicators, { type = "sidePlusBright",  order = 16.3, name = L["Bottom Side (right)"] })
table.insert(indicators, { type = "sidePlusLtop",    order = 17.1, name = L["Left Side (top)"] })
table.insert(indicators, { type = "sidePlusLcenter", order = 17.2, name = L["Left Side (center)"] })
table.insert(indicators, { type = "sidePlusLbottom", order = 17.3, name = L["Left Side (bottom)"] })
table.insert(indicators, { type = "sidePlusRtop",    order = 18.1, name = L["Right Side (top)"] })
table.insert(indicators, { type = "sidePlusRcenter", order = 18.2, name = L["Right Side (center)"] })
table.insert(indicators, { type = "sidePlusRbottom", order = 18.3, name = L["Right Side (bottom)"] })

local statusmap = GridFrame.db.profile.statusmap
if not statusmap["sidePlusTleft"] then
	statusmap["sidePlusTleft"] = {}
	statusmap["sidePlusTcenter"] = {}
	statusmap["sidePlusTright"] = {}
	statusmap["sidePlusBleft"] = {}
	statusmap["sidePlusBcenter"] = {}
	statusmap["sidePlusBright"] = {}
	statusmap["sidePlusLtop"] = {}
	statusmap["sidePlusLcenter"] = {}
	statusmap["sidePlusLbottom"] = {}
	statusmap["sidePlusRtop"] = {}
	statusmap["sidePlusRcenter"] = {}
	statusmap["sidePlusRbottom"] = {}
end

local GridIndicatorSidePlusFrameClass = AceOO.Class(GridFrame.frameClass)

local _frameClass = nil
if not _frameClass then
	_frameClass = GridFrame.frameClass
	GridFrame.frameClass = GridIndicatorSidePlusFrameClass
end

function GridIndicatorSidePlus:OnEnable()
	self:CleanOptionsMenu()	-- hack for better indicator menu

	GridFrame.options.args.advanced.args["sideplus"] = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-TRBL",
		name = L["Side Plus"],
		desc = L["Options for Side Plus."],
		order = 109,
		args = {
			["configuration"] = {
				type = "toggle",
				name = L["Configuration Mode"],
				desc = L["Shows all Side Plus indicators."],
				order = 10,
				get = function()
					return configMode
				end,
				set = function(v) 
					configMode = v
					GridFrame:WithAllFrames(function(f) f:GridIndicatorSidePlusConfig() end)
				end
			},
			["header1"] = {
				type = "header",
				order = 15,
				name = " ",
			},
			["sideplusspace"] = {
				type = "range",
				name = L["Side Plus Space"],
				desc = L["Adjust the space between the Side Plus indicators."],
				order = 20,
				min = 0,
				max = 20,
				step = 1,
				get = function()
					return GridIndicatorSidePlus.db.profile.SidePlusSpace
				end,
				set = function(v)
					GridIndicatorSidePlus.db.profile.SidePlusSpace = v
					GridFrame:WithAllFrames(function(f) f:SetCornerSize(GridFrame.db.profile.cornerSize) end)
				end,
			},
			["sideplusoffset"] = {
				type = "range",
				name = L["Side Plus Offset"],
				desc = L["Adjust the offset for the Side Plus indicators."],
				order = 30,
				min = -10,
				max = 10,
				step = 1,
				get = function()
					return GridIndicatorSidePlus.db.profile.SidePlusOffset
				end,
				set = function(v)
					GridIndicatorSidePlus.db.profile.SidePlusOffset = v
					GridFrame:WithAllFrames(function(f) f:SetCornerSize(GridFrame.db.profile.cornerSize) end)
				end,
			},
			["sideplusoriginalsize"] = {
				type = "toggle",
				name = L["Same size as Grid standard"],
				desc = L["If enabled, the size of the Side Plus indicator is adjustable with the standard Grid corner option. If deactivated, you can set an individual size for the Side Plus indicators."],
				order = 40,
				get = function()
					return GridIndicatorSidePlus.db.profile.SidePlusOriginalSize
				end,
				set = function(v)
					GridIndicatorSidePlus.db.profile.SidePlusOriginalSize = v
					GridFrame.options.args.advanced.args["sideplus"].args["sideplussize"].disabled = v
					GridFrame:WithAllFrames(function(f) f:SetCornerSize(GridFrame.db.profile.cornerSize) end)
				end,
			},
			["sideplussize"] = {
				type = "range",
				name = L["Side Plus Size"],
				desc = L["Adjust the size of the Side Plus indicators."],
				order = 50,
				disabled = GridIndicatorSidePlus.db.profile.SidePlusOriginalSize,
				min = 3,
				max = 20,
				step = 1,
				get = function()
					return GridIndicatorSidePlus.db.profile.SidePlusSize
				end,
				set = function(v)
					GridIndicatorSidePlus.db.profile.SidePlusSize = v
					GridFrame:WithAllFrames(function(f) f:SetCornerSize(GridFrame.db.profile.cornerSize) end)
				end,
			}
		}
	}

	hooksecurefunc(GridFrame, "UpdateOptionsMenu", GridIndicatorSidePlus.CleanOptionsMenu) -- hack for better indicator menu
	self:RegisterEvent("Grid_Enabled")
	self:RegisterEvent("Grid_Disabled")
end

function GridIndicatorSidePlus:Grid_Enabled()
	configMode = false
end

function GridIndicatorSidePlus:Grid_Disabled()
	configMode = false
end

function GridIndicatorSidePlus:CleanOptionsMenu()
	if not GridFrame.options.args.sidePlusTleft then return end
	if not Grid:GetModule("GridStatus"):IsModuleActive("GridIndicatorSidePlus") then return end

	GridFrame.options.args["sideplusT"] = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-T",
		name = L["Top Side"],
		desc = L["Options for Top Side indicators."],
		order = 65.1,
		args = {
			["sidePlusTleft"]   = GridFrame.options.args.sidePlusTleft,
			["sidePlusTcenter"] = GridFrame.options.args.sidePlusTcenter,
			["sidePlusTright"]  = GridFrame.options.args.sidePlusTright
		}
	}
	GridFrame.options.args["sideplusT"].args["sidePlusTleft"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tleft"
	GridFrame.options.args["sideplusT"].args["sidePlusTcenter"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tcenter"
	GridFrame.options.args["sideplusT"].args["sidePlusTright"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Tright"

	GridFrame.options.args.sidePlusTleft = nil
	GridFrame.options.args.sidePlusTcenter = nil
	GridFrame.options.args.sidePlusTright = nil

	GridFrame.options.args["sideplusB"] = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-B",
		name = L["Bottom Side"],
		desc = L["Options for Bottom Side indicators."],
		order = 66.1,
		args = {
			["sidePlusBleft"]   = GridFrame.options.args.sidePlusBleft,
			["sidePlusBcenter"] = GridFrame.options.args.sidePlusBcenter,
			["sidePlusBright"]  = GridFrame.options.args.sidePlusBright
		}
	}
	GridFrame.options.args["sideplusB"].args["sidePlusBleft"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bleft"
	GridFrame.options.args["sideplusB"].args["sidePlusBcenter"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bcenter"
	GridFrame.options.args["sideplusB"].args["sidePlusBright"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Bright"

	GridFrame.options.args.sidePlusBleft = nil
	GridFrame.options.args.sidePlusBcenter = nil
	GridFrame.options.args.sidePlusBright = nil

	GridFrame.options.args["sideplusL"] = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-L",
		name = L["Left Side"],
		desc = L["Options for Left Side indicators."],
		order = 67.1,
		args = {
			["sidePlusLtop"]    = GridFrame.options.args.sidePlusLtop,
			["sidePlusLcenter"] = GridFrame.options.args.sidePlusLcenter,
			["sidePlusLbottom"] = GridFrame.options.args.sidePlusLbottom
		}
	}
	GridFrame.options.args["sideplusL"].args["sidePlusLtop"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Ltop"
	GridFrame.options.args["sideplusL"].args["sidePlusLcenter"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Lcenter"
	GridFrame.options.args["sideplusL"].args["sidePlusLbottom"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Lbottom"

	GridFrame.options.args.sidePlusLtop = nil
	GridFrame.options.args.sidePlusLcenter = nil
	GridFrame.options.args.sidePlusLbottom = nil

	GridFrame.options.args["sideplusR"] = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-R",
		name = L["Right Side"],
		desc = L["Options for Right Side indicators."],
		order = 68.1,
		args = {
			["sidePlusRtop"]    = GridFrame.options.args.sidePlusRtop,
			["sidePlusRcenter"] = GridFrame.options.args.sidePlusRcenter,
			["sidePlusRbottom"] = GridFrame.options.args.sidePlusRbottom
		}
	}
	GridFrame.options.args["sideplusR"].args["sidePlusRtop"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rtop"
	GridFrame.options.args["sideplusR"].args["sidePlusRcenter"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rcenter"
	GridFrame.options.args["sideplusR"].args["sidePlusRbottom"].icon = "Interface\\AddOns\\GridIndicatorSidePlus\\GridIndicatorSidePlus-icon-Rbottom"

	GridFrame.options.args.sidePlusRtop = nil
	GridFrame.options.args.sidePlusRcenter = nil
	GridFrame.options.args.sidePlusRbottom = nil
end

function GridIndicatorSidePlusFrameClass.prototype:GridIndicatorSidePlusConfig()
	if configMode then
		self:SetIndicator("sidePlusTleft",   {r=0.89, g=0.57, b=0.27, a=1})
		self:SetIndicator("sidePlusTcenter", {r=0.31, g=0.62, b=0.86, a=1})
		self:SetIndicator("sidePlusTright",  {r=0.74, g=0.34, b=0.49, a=1})
		self:SetIndicator("sidePlusBleft",   {r=0.12, g=0.46, b=0.97, a=1})
		self:SetIndicator("sidePlusBcenter", {r=0.77, g=0.02, b=0.24, a=1})
		self:SetIndicator("sidePlusBright",  {r=0.17, g=0.65, b=0.33, a=1})
		self:SetIndicator("sidePlusLtop",    {r=0.56, g=0.74, b=0.81, a=1})
		self:SetIndicator("sidePlusLcenter", {r=0.87, g=0.71, b=0.06, a=1})
		self:SetIndicator("sidePlusLbottom", {r=0.29, g=0.83, b=0.36, a=1})
		self:SetIndicator("sidePlusRtop",    {r=0.59, g=0.31, b=0.40, a=1})
		self:SetIndicator("sidePlusRcenter", {r=0.42, g=0.77, b=0.11, a=1})
		self:SetIndicator("sidePlusRbottom", {r=0.33, g=0.65, b=0.37, a=1})
	else
		self:ClearIndicator("sidePlusTleft")
		self:ClearIndicator("sidePlusTcenter")
		self:ClearIndicator("sidePlusTright")
		self:ClearIndicator("sidePlusBleft")
		self:ClearIndicator("sidePlusBcenter")
		self:ClearIndicator("sidePlusBright")
		self:ClearIndicator("sidePlusLtop")
		self:ClearIndicator("sidePlusLcenter")
		self:ClearIndicator("sidePlusLbottom")
		self:ClearIndicator("sidePlusRtop")
		self:ClearIndicator("sidePlusRcenter")
		self:ClearIndicator("sidePlusRbottom")
		GridFrame:UpdateAllFrames()
	end
end

function GridIndicatorSidePlusFrameClass.prototype:CreateIndicator(indicator)
	GridIndicatorSidePlusFrameClass.super.prototype.CreateIndicator(self, indicator)
	local f = self.frame
	local wh = GridIndicatorSidePlus.db.profile.SidePlusSize
	local space = GridIndicatorSidePlus.db.profile.SidePlusSpace
	local offset = GridIndicatorSidePlus.db.profile.SidePlusOffset
	if GridIndicatorSidePlus.db.profile.SidePlusOriginalSize then
		wh = GridFrame.db.profile.cornerSize
	end
	if indicator == "sidePlusTleft" then
		f.sidePlusTleft:SetPoint("TOP", f, "TOP", -space, offset*-1)
		f.sidePlusTleft:SetWidth(wh)
		f.sidePlusTleft:SetHeight(wh)
	elseif indicator == "sidePlusTcenter" then
		f.sidePlusTcenter:SetPoint("TOP", f, "TOP", 0, offset*-1)
		f.sidePlusTcenter:SetWidth(wh)
		f.sidePlusTcenter:SetHeight(wh)
	elseif indicator == "sidePlusTright" then
		f.sidePlusTright:SetPoint("TOP", f, "TOP", space, offset*-1)
		f.sidePlusTright:SetWidth(wh)
		f.sidePlusTright:SetHeight(wh)
	elseif indicator == "sidePlusBleft" then
		f.sidePlusBleft:SetPoint("BOTTOM", f, "BOTTOM", -space, offset)
		f.sidePlusBleft:SetWidth(wh)
		f.sidePlusBleft:SetHeight(wh)
	elseif indicator == "sidePlusBcenter" then
		f.sidePlusBcenter:SetPoint("BOTTOM", f, "BOTTOM", 0, offset)
		f.sidePlusBcenter:SetWidth(wh)
		f.sidePlusBcenter:SetHeight(wh)
	elseif indicator == "sidePlusBright" then
		f.sidePlusBright:SetPoint("BOTTOM", f, "BOTTOM", space, offset)
		f.sidePlusBright:SetWidth(wh)
		f.sidePlusBright:SetHeight(wh)
	elseif indicator == "sidePlusLtop" then
		f.sidePlusLtop:SetPoint("LEFT", f, "LEFT", offset, space)
		f.sidePlusLtop:SetWidth(wh)
		f.sidePlusLtop:SetHeight(wh)
	elseif indicator == "sidePlusLcenter" then
		f.sidePlusLcenter:SetPoint("LEFT", f, "LEFT", offset, 0)
		f.sidePlusLcenter:SetWidth(wh)
		f.sidePlusLcenter:SetHeight(wh)
	elseif indicator == "sidePlusLbottom" then
		f.sidePlusLbottom:SetPoint("LEFT", f, "LEFT", offset, -space)
		f.sidePlusLbottom:SetWidth(wh)
		f.sidePlusLbottom:SetHeight(wh)
	elseif indicator == "sidePlusRtop" then
		f.sidePlusRtop:SetPoint("RIGHT", f, "RIGHT", offset*-1, space)
		f.sidePlusRtop:SetWidth(wh)
		f.sidePlusRtop:SetHeight(wh)
	elseif indicator == "sidePlusRcenter" then
		f.sidePlusRcenter:SetPoint("RIGHT", f, "RIGHT", offset*-1, 0)
		f.sidePlusRcenter:SetWidth(wh)
		f.sidePlusRcenter:SetHeight(wh)
	elseif indicator == "sidePlusRbottom" then
		f.sidePlusRbottom:SetPoint("RIGHT", f, "RIGHT", offset*-1, -space)
		f.sidePlusRbottom:SetWidth(wh)
		f.sidePlusRbottom:SetHeight(wh)
	end
end

function GridIndicatorSidePlusFrameClass.prototype:SetIndicator(indicator, color, text, value, maxValue, texture, start, duration, stack)
	GridIndicatorSidePlusFrameClass.super.prototype.SetIndicator(self, indicator, color, text, value, maxValue, texture, start, duration, stack)
	if indicator == "sidePlusTleft"
	or indicator == "sidePlusTcenter"
	or indicator == "sidePlusTright"
	or indicator == "sidePlusBleft"
	or indicator == "sidePlusBcenter"
	or indicator == "sidePlusBright"
	or indicator == "sidePlusLtop"
	or indicator == "sidePlusLcenter"
	or indicator == "sidePlusLbottom"
	or indicator == "sidePlusRtop"
	or indicator == "sidePlusRcenter"
	or indicator == "sidePlusRbottom"
	then
		if not self.frame[indicator] then
			self:CreateIndicator(indicator)
		end
		if not color then color = { r = 1, g = 1, b = 1, a = 1 } end
		self.frame[indicator]:SetBackdropColor(color.r, color.g, color.b, color.a)
		self.frame[indicator]:Show()
	end
end

function GridIndicatorSidePlusFrameClass.prototype:ClearIndicator(indicator)
	GridIndicatorSidePlusFrameClass.super.prototype.ClearIndicator(self, indicator)
	if indicator == "sidePlusTleft"
	or indicator == "sidePlusTcenter"
	or indicator == "sidePlusTright"
	or indicator == "sidePlusBleft"
	or indicator == "sidePlusBcenter"
	or indicator == "sidePlusBright"
	or indicator == "sidePlusLtop"
	or indicator == "sidePlusLcenter"
	or indicator == "sidePlusLbottom"
	or indicator == "sidePlusRtop"
	or indicator == "sidePlusRcenter"
	or indicator == "sidePlusRbottom"
	then
		if self.frame[indicator] then
			self.frame[indicator]:SetBackdropColor(1, 1, 1, 1)
			self.frame[indicator]:Hide()
		end
	end
end

function GridIndicatorSidePlusFrameClass.prototype:SetCornerSize(size)
	GridIndicatorSidePlusFrameClass.super.prototype.SetCornerSize(self, GridFrame.db.profile.cornerSize)
	local f = self.frame
	local wh = GridIndicatorSidePlus.db.profile.SidePlusSize
	local space = GridIndicatorSidePlus.db.profile.SidePlusSpace
	local offset = GridIndicatorSidePlus.db.profile.SidePlusOffset
	if GridIndicatorSidePlus.db.profile.SidePlusOriginalSize then
		wh = GridFrame.db.profile.cornerSize
	end
	if f.sidePlusTleft then
		f.sidePlusTleft:SetPoint("TOP", f, "TOP", -space, offset*-1)
		f.sidePlusTleft:SetWidth(wh)
		f.sidePlusTleft:SetHeight(wh)
	end
	if f.sidePlusTcenter then
		f.sidePlusTcenter:SetPoint("TOP", f, "TOP", 0, offset*-1)
		f.sidePlusTcenter:SetWidth(wh)
		f.sidePlusTcenter:SetHeight(wh)
	end
	if f.sidePlusTright then
		f.sidePlusTright:SetPoint("TOP", f, "TOP", space, offset*-1)
		f.sidePlusTright:SetWidth(wh)
		f.sidePlusTright:SetHeight(wh)
	end
	if f.sidePlusBleft then
		f.sidePlusBleft:SetPoint("BOTTOM", f, "BOTTOM", -space, offset)
		f.sidePlusBleft:SetWidth(wh)
		f.sidePlusBleft:SetHeight(wh)
	end
	if f.sidePlusBcenter then
		f.sidePlusBcenter:SetPoint("BOTTOM", f, "BOTTOM", 0, offset)
		f.sidePlusBcenter:SetWidth(wh)
		f.sidePlusBcenter:SetHeight(wh)
	end
	if f.sidePlusBright then
		f.sidePlusBright:SetPoint("BOTTOM", f, "BOTTOM", space, offset)
		f.sidePlusBright:SetWidth(wh)
		f.sidePlusBright:SetHeight(wh)
	end
	if f.sidePlusLtop then
		f.sidePlusLtop:SetPoint("LEFT", f, "LEFT", offset, space)
		f.sidePlusLtop:SetWidth(wh)
		f.sidePlusLtop:SetHeight(wh)
	end
	if f.sidePlusLcenter then
		f.sidePlusLcenter:SetPoint("LEFT", f, "LEFT", offset, 0)
		f.sidePlusLcenter:SetWidth(wh)
		f.sidePlusLcenter:SetHeight(wh)
	end
	if f.sidePlusLbottom then
		f.sidePlusLbottom:SetPoint("LEFT", f, "LEFT", offset, -space)
		f.sidePlusLbottom:SetWidth(wh)
		f.sidePlusLbottom:SetHeight(wh)
	end
	if f.sidePlusRtop then
		f.sidePlusRtop:SetPoint("RIGHT", f, "RIGHT", offset*-1, space)
		f.sidePlusRtop:SetWidth(wh)
		f.sidePlusRtop:SetHeight(wh)
	end
	if f.sidePlusRcenter then
		f.sidePlusRcenter:SetPoint("RIGHT", f, "RIGHT", offset*-1, 0)
		f.sidePlusRcenter:SetWidth(wh)
		f.sidePlusRcenter:SetHeight(wh)
	end
	if f.sidePlusRbottom then
		f.sidePlusRbottom:SetPoint("RIGHT", f, "RIGHT", offset*-1, -space)
		f.sidePlusRbottom:SetWidth(wh)
		f.sidePlusRbottom:SetHeight(wh)
	end
end